import React, { useEffect, useState } from "react";
import { FlightCreate, FlightCreateFull, Flight } from "../../Types";
import { useNavigate } from "react-router";
import "./FlightForm.css";
import { createFlight } from "../../Api";

export const CreateFlight = () => {
    const navigate = useNavigate();
    const [error, setError] = useState<string | null>(null);
    const [flight, setFlight] = useState<Flight>();

    const [flightData, setFlightData] = useState<FlightCreate>({
        flight_code: "ABC123",
        status: "Scheduled",
        origin: "Frankfurt",
        destination: "Rome",
        departure_time: new Date().toISOString().slice(0, 16),
        arrival_time: new Date(new Date().getTime() + 2 * 60 * 60 * 1000).toISOString().slice(0, 16),
        gate: "A1",
    });

    const handleSubmit = async (event: React.FormEvent) => {
        event.preventDefault();

        if (flightData.departure_time >= flightData.arrival_time) {
            setError("Departure time must be before the arrival time")
            return;
        }

        const token = localStorage.getItem("token");
        if (!token) {
            setError("No token!");
            return;
        }

        let newFlightData: FlightCreate = {
            flight_code: flightData.flight_code,
            status: flightData.status,
            origin: flightData.origin,
            destination: flightData.destination,
            departure_time: formatDate(flightData.departure_time),
            arrival_time: formatDate(flightData.arrival_time),
            gate: flightData.gate,
        }

        createFlight(newFlightData, token)
            .then((data) => {
                setFlight(data);
                navigate("/flights")
            })
            .catch((error) => {
                try {
                    setError(JSON.parse(error.response.data)["detail"] as string);
                } catch { }
            });
    };

    function formatDate(isoString: string) {
        let date = new Date(isoString)
        const year = date.getFullYear();
        const month = (date.getMonth() + 1).toString().padStart(2, '0');
        const day = date.getDate().toString().padStart(2, '0');
        let hours = date.getHours();
        const minutes = date.getMinutes().toString().padStart(2, '0');
        const amOrPm = hours >= 12 ? 'PM' : 'AM';

        hours = hours % 12 || 12;

        return `${year}-${month}-${day} ${hours.toString().padStart(2, '0')}:${minutes} ${amOrPm}`;
    }

    return (
        <form onSubmit={handleSubmit}>
            {error &&
                <div className="form-error">
                    <p><strong>Error:</strong> {error}</p>
                </div>
            }
            <label>
                Flight Code:
                <input
                    type="text"
                    value={flightData.flight_code}
                    name="Code"
                    onChange={(e) =>
                        setFlightData({ ...flightData, flight_code: e.target.value })
                    }
                />
            </label>
            <label>
                Status:
                <input
                    type="text"
                    name="Status"
                    value={flightData.status}
                    onChange={(e) =>
                        setFlightData({ ...flightData, status: e.target.value })
                    }
                />
            </label>
            <label>
                Origin:
                <input
                    type="text"
                    value={flightData.origin}
                    name="Origin"
                    onChange={(e) =>
                        setFlightData({ ...flightData, origin: e.target.value })
                    }
                />
            </label>
            <label>
                Destination:
                <input
                    type="text"
                    name="Destination"
                    value={flightData.destination}
                    onChange={(e) =>
                        setFlightData({ ...flightData, destination: e.target.value })
                    }
                />
            </label>
            <label>
                Departure Time:
                <input
                    type="datetime-local"
                    name="DepartureTime"
                    value={flightData.departure_time}
                    onChange={(e) =>
                        setFlightData({ ...flightData, departure_time: e.target.value })
                    }
                />
            </label>
            <label>
                Arrival Time:
                <input
                    type="datetime-local"
                    name="ArrivalTime"
                    value={flightData.arrival_time}
                    onChange={(e) =>
                        setFlightData({ ...flightData, arrival_time: e.target.value })
                    }
                />
            </label>
            <label>
                Gate:
                <input
                    type="text"
                    name="Gate"
                    value={flightData.gate}
                    onChange={(e) => setFlightData({ ...flightData, gate: e.target.value })}
                />
            </label>
            <button name="CreateFlightButton" type="submit">Create</button>
        </form>
    );
};
