import React, { useEffect, useState } from "react";
import { Link, useNavigate } from "react-router-dom";
import { Avatar, Space, Typography, Tag, Button, Modal } from "antd";
import { RightOutlined, ClockCircleOutlined, SwapOutlined, EnvironmentOutlined, CalendarOutlined } from "@ant-design/icons";

import "./Card.css";
import { getChatId, getSubscription, subscribeToFlight, unsubscribeFromFlight, editFlight } from "../../../Api";
import { Flight, FlightEdit, User } from "../../../Types";

interface CardProps {
    flight: Flight;
    user: User | undefined;
    subscribed: boolean;
    refresh: any;
    isAirline: boolean;
    isAdmin: boolean;
    refreshFlights: any;
}

const { Text } = Typography;

export const Card: React.FC<CardProps> = ({ flight, user, subscribed, refresh, refreshFlights, isAirline, isAdmin }) => {
    const [modalVisible, setModalVisible] = useState<boolean>(false);

    const navigate = useNavigate();

    const handleSubscribe = async (event: React.FormEvent) => {
        event.preventDefault();

        const token = localStorage.getItem("token");
        if (!token || !user) {
            return;
        }

        const data = {
            user_id: user.id,
            flight_id: flight.id
        }

        subscribeToFlight(data, token)
            .then(() => {
                refresh()
                getChatId(user.id, token)
                    .then(() => { })
                    .catch((error) => {
                        setModalVisible(true);
                    })
            })
            .catch((error) => {
            });
    };

    const handleEdit = async (event: React.FormEvent) => {
        event.preventDefault();
        navigate(`/edit-flight/${flight.id}`);
    };

    const handleDelete = async (event: React.FormEvent) => {
        event.preventDefault();

        const token = localStorage.getItem("token");
        if (!token || !user) {
            return;
        }

        let data: any = {
            status: "Deleted"
        }

        editFlight("" + flight.id, data, token)
            .then(() => {
                refreshFlights()
            })
            .catch((error) => { });
    };

    const handleModalClose = () => {
        setModalVisible(false);
    };

    const handleUnsubscribe = async (event: React.FormEvent) => {
        event.preventDefault();

        const token = localStorage.getItem("token");
        if (!token || !user) {
            return;
        }

        const data = {
            user_id: user.id,
            flight_id: flight.id
        }

        unsubscribeFromFlight(data, token)
            .then(() => {
                refresh()
            })
            .catch((error) => { });
    };

    return (
        <div className="flight-card">
            <Space size={8} align="center">
                <div>
                    <Text strong>{flight.flight_code} </Text>
                    <Text type="secondary">
                        {flight.origin} <SwapOutlined /> {flight.destination}
                    </Text>
                </div>
            </Space>
            <div className="flight-details">
                <Space size={8} direction="horizontal">
                    <Text strong>Status:</Text>
                    <Tag color={flight.status === "Scheduled" ? "green" : "orange"}>{flight.status}</Tag>
                </Space>
                <Space size={8} direction="horizontal">
                    <Text strong>Departure:</Text>
                    <Space size={2} align="baseline">
                        <Text>{flight.departure_time}</Text>
                    </Space>
                </Space>
                <Space size={8} direction="horizontal">
                    <Text strong>Arrival:</Text>
                    <Space size={2} align="baseline">
                        <Text>{flight.arrival_time}</Text>
                    </Space>
                </Space>
                <Space size={8} direction="horizontal">
                    <Text strong>Gate:</Text>
                    <Text>{flight.gate}</Text>
                </Space>
                <Space size={8} direction="horizontal">
                    <Text strong>Flight ID:</Text>
                    <Text>{flight.id}</Text>
                </Space>
                {(user && ((!isAirline && !isAdmin) || ((flight.user_id == user.id) || isAdmin))) && <br></br>}
            </div>
            {user && (!isAirline && !isAdmin ?
                (
                    !(subscribed) ?
                        <Space size={8} direction="horizontal" style={{ justifyContent: "center" }}>
                            <Button type="primary" onClick={handleSubscribe}>
                                Subscribe
                            </Button>
                        </Space>
                        :
                        <Space size={8} direction="horizontal" style={{ justifyContent: "center" }}>
                            <Button type="primary" onClick={handleUnsubscribe}>
                                Unsubscribe
                            </Button>
                        </Space>
                )
                :
                (
                    (flight.user_id == user.id) || isAdmin ?
                        <Space size={8} direction="horizontal" style={{ justifyContent: "center" }}>
                            <Button type="primary" onClick={handleEdit}>
                                Edit
                            </Button>
                            <Button type="primary" onClick={handleDelete}>
                                Delete
                            </Button>
                        </Space>
                        :
                        <></>
                )
            )
            }
            <Modal
                title="Attention"
                open={modalVisible}
                onCancel={handleModalClose}
                footer={[
                    <Button key="ok" type="primary" onClick={handleModalClose}>
                        OK
                    </Button>
                ]}
            >
                <p>To start receiving messages open this link on your smartphone:

                </p>
                <Link to={`https://t.me/fids_system_bot?start=${user?.id}`} target="_blank">
                    {`https://t.me/fids_system_bot?start=${user?.id}`}
                </Link>
            </Modal>
        </div>
    );
};
