import React, { useEffect, useState } from "react";
import { Card } from "./Card/Card";
import { useFetchFlights } from "../../hooks/useFetchFlights";
import { Flight } from "../../Types";
import { useNavigate } from "react-router";
import useAuth from "../../useAuth";
import { useFetchSubscriptions } from "../../hooks/useFetchSubscriptions";
import { Button, Input, Space } from "antd";
import { SearchOutlined, CloseCircleOutlined } from '@ant-design/icons';

interface Props {
    flights?: Flight[];
}

export const Home: React.FC<Props> = (props) => {
    const urlParams = new URLSearchParams(window.location.search);
    const initialPage = parseInt(urlParams.get('page') || '1', 10);
    const query = urlParams.get('search');
    const [searchQuery, setSearchQuery] = useState(query);
    const { flights, count, error, isLoading, fetchData: refreshFlights } = useFetchFlights(initialPage, searchQuery);
    const navigate = useNavigate()

    const [currentPage, setCurrentPage] = useState(initialPage);
    const { loading, isAirline, user, token, isAdmin } = useAuth();
    
    const { subscriptions, loading: subsLoading, fetchData } = useFetchSubscriptions(user, token, !isAirline && !isAdmin);

    useEffect(() => {
        const newParams = new URLSearchParams(window.location.search);
        newParams.set('page', currentPage.toString());
        if (searchQuery && searchQuery.length != 0) {
            newParams.set("search", searchQuery);
        }
        navigate(`?${newParams.toString()}`);
    }, [currentPage, navigate]);

    useEffect(() => {
        if (currentPage > 1 && count == 0) {
            setCurrentPage(currentPage - 1);
        }
    }, [count]);

    const goToPrevPage = () => {
        if (currentPage > 1) {
            setCurrentPage(currentPage - 1);
        }
    };

    const goToNextPage = () => {
        setCurrentPage(currentPage + 1);
    };

    const checkMaxPage = () => {
        return currentPage * 8 >= count ? false : true
    }

    const checkMinPage = () => {
        return currentPage > 1 ? true : false
    }

    const handleSearchSubmit = (e: React.FormEvent | undefined) => {
        let searchQueryValue = null

        if (!!e) {
            e.preventDefault();
            const formData = new FormData(e.currentTarget as HTMLFormElement);
            searchQueryValue = formData.get("searchInput") as string;
        }

        setSearchQuery(searchQueryValue)
        const newParams = new URLSearchParams(window.location.search);
        newParams.set('page', "1");
        if (searchQueryValue && searchQueryValue.length != 0) {
            newParams.set("search", searchQueryValue);
        } else {
            newParams.delete("search")
        }
        setCurrentPage(1)
        navigate(`?${newParams.toString()}`);
    };

    if (loading || isLoading) {
        return <div>Loading...</div>;
    }

    let columns = flights.length >= 4 ? 4 : flights.length
    
    return (
        <div className="Box">
            {isAirline ? <button name="CreateFlight" onClick={() => { navigate("/create-flight") }}>Create flight</button> : <></>}
            {isAdmin ? <button onClick={() => { navigate("/create-airline") }}>Create airline user</button> : <></>}
            <form onSubmit={handleSearchSubmit}>
                <Space size={8} direction="horizontal">
                    <div style={{ display: 'flex' }}>
                    <Input
                        type="text"
                        placeholder="Enter flight code"
                        name="searchInput"
                        defaultValue={searchQuery || ""}
                        style={{ borderRadius: 0 }}
                    />
                    {searchQuery && (<Button
                        type="text"
                        onClick={() => handleSearchSubmit(undefined)}
                        style={{ border: 'none', backgroundColor: 'transparent', marginRight: '5px' }}
                    >
                        <CloseCircleOutlined style={{ color: 'gray' }} />
                    </Button>
                    )}
                    <Button type="primary" style={{ border: 'none', borderRadius: 0 }} htmlType="submit">
                        <SearchOutlined/>
                    </Button>
                    </div>
                </Space>
            </form>
            <div className="Items" style={{ gridTemplateColumns: `repeat(${columns}, minmax(80px, 250px))` }}>
                {flights.length == 0 && <p>No flights found!</p>}
                {(props.flights ? props.flights : flights).map((f) => {
                    return <Card key={f.id} flight={f} user={user} 
                    subscribed={subscriptions.some((i => i.flight_id === f.id))} 
                    refresh={fetchData} refreshFlights={refreshFlights} isAirline={isAirline} isAdmin={isAdmin} />;
                })}
                {error ? <div className="Disconnected">{error}</div> : <></>}
            </div>
            <div>
                {checkMinPage() && <button onClick={goToPrevPage}>Prev</button>}
                {flights.length != 0 && <span> Page {currentPage} </span>}
                {checkMaxPage() && <button onClick={goToNextPage}>Next</button>}
            </div>
        </div>
    );
};
