from unittest.mock import ANY, Mock, patch

import pytest
from fastapi import HTTPException
from fastapi.testclient import TestClient

import src.api.routes.auth
from src.api.config import API_AUTH
from src.api.main import app
from src.api.routes.auth import checkAuth

client = TestClient(app)


@pytest.fixture(scope="module")
def mock_request():
    with patch.object(src.api.routes.auth, "request", autospec=True) as m:
        yield m


def test_status_endpoint(mock_request):
    mock_request.return_value = (
        {"role": "user", "id": 1},
        200,
        "Mock headers",
    )

    response = client.get("/auth/status")
    assert response.status_code == 200
    assert response.json() == {"role": "user", "id": 1}
    mock_request.assert_called_once_with(
        f"{API_AUTH}/status",
        "GET",
        headers={"Authorization": "", "x-api-request-id": ANY},
    )


@pytest.fixture(scope="function")
def mock_status():
    with patch.object(src.api.routes.auth, "status", autospec=True) as m:
        yield m


@pytest.mark.asyncio
async def test_checkAuth_with_invalid_roles(mock_status):
    mock_status.return_value = {"id": 1, "role": "airline"}

    request_mock = Mock(state=Mock(request_id="mock_request_id"))
    with pytest.raises(HTTPException) as exc_info:
        await checkAuth(request_mock, roles=["admin"])

    assert exc_info.value.status_code == 403
    assert str(exc_info.value.detail) == "You don't have the required permissions."
    mock_status.assert_called_once_with(request_mock, None)


@pytest.mark.asyncio
async def test_checkAuth_with_valid_user_id(mock_status):
    mock_status.return_value = {"id": 1, "role": "airline"}

    request_mock = Mock(state=Mock(request_id="mock_request_id"))
    response = await checkAuth(request_mock, userId="1")

    assert response is None
    mock_status.assert_called_once_with(request_mock, None)


@pytest.mark.asyncio
async def test_checkAuth_with_invalid_user_id(mock_status):
    mock_status.return_value = {"id": 1, "role": "airline"}

    request_mock = Mock(state=Mock(request_id="mock_request_id"))
    with pytest.raises(HTTPException) as exc_info:
        await checkAuth(request_mock, userId="2")

    assert exc_info.value.status_code == 403
    assert str(exc_info.value.detail) == "You don't have the required permissions."
    mock_status.assert_called_once_with(request_mock, None)
