from unittest.mock import ANY, patch

import pytest
from fastapi.testclient import TestClient

import src.api.routes.auth
import src.api.routes.flights
from src.api.config import API_FLIGHTS
from src.api.main import app

client = TestClient(app)


@pytest.fixture(scope="function")
def mock_request():
    with patch.object(src.api.routes.flights, "request", autospec=True) as m:
        yield m


def test_get_flight_by_id(mock_request):
    mock_request.return_value = (
        {
            "id": 1,
            "flight_code": "AOZ789",
            "status": "Scheduled",
            "origin": "Frankfurt",
            "destination": "Paris",
            "departure_time": "2023-11-04 07:58 PM",
            "arrival_time": "2023-11-05 03:43 AM",
            "gate": "X8",
            "user_id": 1,
        },
        200,
        "Mock headers",
    )

    response = client.get("/flights/1")
    assert response.status_code == 200
    assert response.json() == {
        "id": 1,
        "flight_code": "AOZ789",
        "status": "Scheduled",
        "origin": "Frankfurt",
        "destination": "Paris",
        "departure_time": "2023-11-04 07:58 PM",
        "arrival_time": "2023-11-05 03:43 AM",
        "gate": "X8",
        "user_id": 1,
    }
    mock_request.assert_called_once_with(
        f"{API_FLIGHTS}/1", "GET", headers={"x-api-request-id": ANY}
    )


@pytest.fixture(scope="function")
def mock_status():
    with patch.object(src.api.routes.auth, "status", autospec=True) as m:
        yield m


def test_create_flight(mock_request, mock_status):
    mock_status.return_value = {"id": 1, "role": "airline"}
    mock_request.return_value = (
        {
            "id": 1,
            "flight_code": "AOZ789",
            "status": "Scheduled",
            "origin": "Frankfurt",
            "destination": "Paris",
            "departure_time": "2023-11-04 07:58 PM",
            "arrival_time": "2023-11-05 03:43 AM",
            "gate": "X8",
        },
        200,
        "Mock headers",
    )

    response = client.post(
        "/flights",
        json={
            "id": 1,
            "flight_code": "AOZ789",
            "status": "Scheduled",
            "origin": "Frankfurt",
            "destination": "Paris",
            "departure_time": "2023-11-04 07:58 PM",
            "arrival_time": "2023-11-05 03:43 AM",
            "gate": "X8",
        },
        headers={"Authorization": "mock_token"},
    )
    assert response.status_code == 200
    assert response.json() == {
        "id": 1,
        "flight_code": "AOZ789",
        "status": "Scheduled",
        "origin": "Frankfurt",
        "destination": "Paris",
        "departure_time": "2023-11-04 07:58 PM",
        "arrival_time": "2023-11-05 03:43 AM",
        "gate": "X8",
    }
